/* 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Sablotron XSLT Processor.
 * 
 * The Initial Developer of the Original Code is Ginger Alliance Ltd.
 * Portions created by Ginger Alliance are Copyright (C) 2000 Ginger
 * Alliance Ltd. All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the MPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the MPL or the
 * GPL.
 */

#ifndef ContextHIncl
#define ContextHIncl

/*****************************************************************

    c   o   n   t   e   x   t   .   h

*****************************************************************/

#include "expr.h"
#include "verts.h"

class CList : public VertexList
{
public:
    CList();
    virtual int compare(Vertex *v1, Vertex *v2);
    void incRefCount();
    int decRefCount();
private:
    int refCount;
};


/*****************************************************************

    C   o   n   t   e   x   t

  expression evaluation context

*****************************************************************/

class Context 
{
public:
    Context();
    ~Context();
    int 
        getPosition() const;
    int 
        getSize() const;
    Vertex 
        *current() const;
    Vertex
        *getCurrentNode() const;
    void
        setCurrentNode(Vertex *);
    void    
        reset();
    Bool 
        isFinished() const;
    Bool
        isVoid() const;
    Vertex 
        *shift();
    void    
        set(Vertex *);
    void 
        setVirtual(Vertex *v, int virtualPosition_, int virtualSize_);
    Vertex
        *operator[] (int) const;
    void
        deppendall();
    void
        append(Vertex *);
    void
        deppend();
    Context
        *copy();
    Context 
        *swallow(Context *);
    void
        swap(int, int);
    Bool
        contains(const Vertex *v) const;
    void sort()
    {
        array -> sort();
    }
    void uniquize();
protected:
    CList 
        *array;
    Vertex
        *currentNode;
    int
        position,
        virtualPosition,
        virtualSize;
};

#endif  //ifndef ContextHIncl
